#include <SDL2/SDL.h>
#include "anim.h"
#include "system.h"
#include "body.h"


void Anim::init(){

    SDL_Init(SDL_INIT_VIDEO);              // Initialize SDL2

}

void Anim::create(){

    window = SDL_CreateWindow("Animation Window",   // window title
                            10,                     // initial x position
                            100,                    // initial y position
                            width,                  // width, in pixels
                            height,                 // height, in pixels
                            0                       // flags
                            );
    renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED);

    // The window is open: enter program loop (see SDL_PollEvent)
    // in loop it will be while(!done). Loop ends once simulation is
    // finished (i.e. done is set to 1). But yet, sim is not done:
    done = 0;

}

void Anim::destroy(){

    // Close and destroy the window
    SDL_DestroyWindow(window);
    SDL_DestroyRenderer(renderer);

    // Clean up
    SDL_Quit();

}


void Anim::doRender(SDL_Renderer *renderer, Object*object, int timestep)
{
    //set the drawing color to black
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);

    //Clear the screen (to blue)
    SDL_RenderClear(renderer);

    //Koordinatenachsen
    //set the drawing color to grey
    SDL_SetRenderDrawColor(renderer, 80, 80, 80, 255);
    SDL_RenderDrawLine(renderer, 30, ORIGy, width-30, ORIGy);         //x-Achse
    //SDL_RenderDrawLine(renderer, ORIGyx, height-30, ORIGx, 30);    //y-Achse

    //set the drawing color to white
    SDL_SetRenderDrawColor(renderer, 255, 255, 255, 255);

    for(int i = 0; i < Ntot; i++){

		SDL_SetRenderDrawColor(renderer, object[i].color[0], object[i].color[1], object[i].color[2], 255);
        DrawCircle(renderer, (object+i)->pxl_x, (object+i)->pxl_y, (object+i)->anim_radius);

        if(object[i].type != ASTROID && timestep > 2){
        	//for(int j = 0; j < (object[i].orbit_size-1); j++){
        	for(int j = 0; j < timestep-1; j++){

        		SDL_RenderDrawLine(renderer, object[i].orbit_x[j], object[i].orbit_y[j], object[i].orbit_x[j+1], object[i].orbit_y[j+1]);
        		//printf("%f\n", object[1].orbit_x[j]);
        		//printf("%f\n", object[10].orbit_x[j]);
        	}
        }
    }

    //SDL_Rect rect = { object1->x, object1->y, object1->a, object1->b };
    //SDL_RenderFillRect(renderer, &rect);

    //We are done drawing, "present" or show to the screen what we've drawn
    SDL_RenderPresent(renderer);
}




int Anim::processEvents(SDL_Window *window, Object *object1)
{
    SDL_Event event;
    int done = 0;

    while(SDL_PollEvent(&event)){
        switch(event.type){
              case SDL_WINDOWEVENT_CLOSE:
              {
                if(window){
                  SDL_DestroyWindow(window);
                  window = NULL;
                  done = 1;
                }
              }
              break;

              case SDL_KEYDOWN:
              {
                switch(event.key.keysym.sym){
                  case SDLK_ESCAPE:
                    done = 1;
                    break;
                }
              }
              break;
              case SDL_QUIT:
                //quit out of the game
                done = 1;
              break;
         }
    }

    const Uint8 *state = SDL_GetKeyboardState(NULL);
    if(state[SDL_SCANCODE_LEFT])
    {
    object1->pxl_x -= 10;
    }
    if(state[SDL_SCANCODE_RIGHT])
    {
    object1->pxl_x += 10;
    }
    if(state[SDL_SCANCODE_UP])
    {
    object1->pxl_y -= 10;
    }
    if(state[SDL_SCANCODE_DOWN])
    {
    object1->pxl_y += 10;
    }


    return done;
}




//void Anim::DrawCircle(SDL_Renderer *renderer, int x, int y, int radius, SDL_Color color)
//{
//    SDL_SetRenderDrawColor(renderer, color.r, color.g, color.b, color.a);
//    for (int w = 0; w < radius * 2; w++)
//    {
//        for (int h = 0; h < radius * 2; h++)
//        {
//            int dx = radius - w; // horizontal offset
//            int dy = radius - h; // vertical offset
//            if ((dx*dx + dy*dy) <= (radius * radius))
//            {
//                SDL_RenderDrawPoint(renderer, x + dx, y + dy);
//            }
//        }
//    }
//}



void Anim::DrawCircle(SDL_Renderer * renderer, int32_t centreX, int32_t centreY, int32_t radius)
{
   const int32_t diameter = (radius * 2);

   int32_t x = (radius - 1);
   int32_t y = 0;
   int32_t tx = 1;
   int32_t ty = 1;
   int32_t error = (tx - diameter);

   while (x >= y)
   {
      //  Each of the following renders an octant of the circle
      SDL_RenderDrawPoint(renderer, centreX + x, centreY - y);
      SDL_RenderDrawPoint(renderer, centreX + x, centreY + y);
      SDL_RenderDrawPoint(renderer, centreX - x, centreY - y);
      SDL_RenderDrawPoint(renderer, centreX - x, centreY + y);
      SDL_RenderDrawPoint(renderer, centreX + y, centreY - x);
      SDL_RenderDrawPoint(renderer, centreX + y, centreY + x);
      SDL_RenderDrawPoint(renderer, centreX - y, centreY - x);
      SDL_RenderDrawPoint(renderer, centreX - y, centreY + x);

      if (error <= 0)
      {
         ++y;
         error += ty;
         ty += 2;
      }

      if (error > 0)
      {
         --x;
         tx += 2;
         error += (tx - diameter);
      }
   }
}
