#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "body.h"
#include "system.h"
#include "anim.h"
#include <mpi.h>

#include <SDL2/SDL.h>

double delta_t = 1e-3;
int MAX_SETPS = 1e6;

int main(int argc, char *argv[]) {

    // ------- MPI Routine ------- //
    // unique rank is asined to each process in a communicator
    int rank;
    // Total number of ranks
    int num_proc;
    // the machine we are on
    char name[80];
    // length of machine name
    int length;
    // Initialize the MPI execution environment
    MPI_Init(&argc, &argv);
    // get this process' rank (process within a communicator)
    // MPI_COMM_WORLD is the default communicator
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    // get the total number of ranks in this communicator
    MPI_Comm_size(MPI_COMM_WORLD, &num_proc);
    // get the name of the processor
    // implementaion specific (may be get hostname, username or systeminfo)
    MPI_Get_processor_name(name, &length);

    /* Deklariere System hier. Damit wird auf jedem Prozessor Speicher für
    System deklariert.
    Prozessor 0 nutzt dies um tatsächlich das System zu
    speichern und mit Daten zu füllen und upzudaten. Somit kann Prozessor 0
    dann auch die Daten rendern und visualisieren. Da nur Prozessor 0
    die Daten visualisiert, brauch auch nur hinter if rank = 0 Anim anim
    deklariert werden.
    Für die anderen Prozessren wird hier an dieser Stelle nun auch System
    deklariert. Dies wird aber auf den anderen Prozessoren nur getan, um
    Speicher bereitzustellen, in den die empfangenen Daten der übertragenen
    Zustandsvektoren über MPI_Recv(...) einsortiert werden kann, um dann direkt
    die FUnktionen sys.attraction und sys.Euler etc. nutzen zu können, um
    die notw. Berechnungen anstellen zu können. Das ist einfacher als auf die
    übertragenen Zustandsvektoren zu operieren. */
    System sys;
    sys.create();
    sys.init();
    sys.belt_init();


    int N = sys.N;

    // allocate storage for state-vector needed for sending
    // and receiving data via MPI_:
    double *y = new double[2*N];
    double *v = new double[2*N];
    double *a = new double[2*N];
    double *pxl = new double[2*N];
    double *mass = new double[N];

    // p = 0;  // num_proc
    // choose correct proc. p to send data of Object i:
    // if(i >= p*N/num_proc){
    //     p++;
    // }

    if(rank == 0){

    // only rank zero contains the system info. Rank zero theirfore
    // disributes work to all other processors and collects update info
    // to updated the system

    // only rank zero collect all data to perform visualization:
    Anim anim;
    anim.init();
    anim.create();

    // start time. Movements start when t = 0.0
    double t = 0.0;

    // // shorter arrays with only entries of interest for proc. p
    // double *y_buffer = new double[2*(N/(num_proc-1) + 1)];
    // double *v_buffer = new double[2*(N/(num_proc-1) + 1)];
    // double *a_buffer = new double[2*(N/(num_proc-1) + 1)];

    int step = 0;

    // while Schleife nur von rank = 0 durchlaufen
    //Event loop
    while(!anim.done){
        anim.start = SDL_GetTicks();        // timing for correct fps

        //Check for events (e.g. move object). If this is uppose to have
        // a physical effect, change Planets->x in processEvents()
        anim.done = anim.processEvents(anim.window, &sys.Objects[0]);

        //Render display
        anim.doRender(anim.renderer, sys.Objects, sys.timestep);

        // time:
        t += delta_t;

        // start when time hits zero: t = 0:
        if(t > 0){
            printf("Step = %d\n", step);


            // fill state-vectors:
            for(int i = 0; i < N; i++){
                y[2*i] = sys.Objects[i].x;
                y[2*i + 1] = sys.Objects[i].y;
                v[2*i] = sys.Objects[i].u;
                v[2*i + 1] = sys.Objects[i].v;
                a[2*i] = sys.Objects[i].ax;
                a[2*i + 1] = sys.Objects[i].ay;
                mass[i] = sys.Objects[i].mass;
            }

            printf("Proc %d step %d: filled state-vectors.\n", rank, step);


            // ------ calc. attraction --------- //

            // to calculate attraction, no matter which proc. will calculate
            // it, they all need the positions and masses of every other
            // Object. So send positoins and mass to every other proc.:
            // distribute data to all processors
            for(int p = 1; p < num_proc; p++){
                printf(">for: send to proc. %d\n", p);
                MPI_Send(y, 2*N, MPI_DOUBLE, p, 0, MPI_COMM_WORLD);
                MPI_Send(mass, N, MPI_DOUBLE, p, 0, MPI_COMM_WORLD);
            }

            printf("Proc %d step %d: sent y and mass from proc %d.\n", rank, step, rank);


            // receive all calculated forces for the timestep:
            // start receiving from proc. 1:
            for(int p = 1; p < num_proc; p++){
                // proc. p calculated forces for the first N/num_proc Obj.
                printf("Proc %d step %d: wait for ax and ay in proc. %d from proc. %d\n", rank, step, rank, p);
                MPI_Recv(a, 2*N, MPI_DOUBLE, p, 0,
                    MPI_COMM_WORLD, MPI_STATUS_IGNORE);

                printf("\nStep %d: Obj[3].ax %lf    Obj[3].ay %lf\n\n", step, a[2*3], a[2*3+1]);

                printf("Proc %d step %d: recv ax and ay in proc. %d from proc. %d\n", rank, step, rank, p);
                // write this into forces of the Obj. N/num_proc Obj.:
                // work was distributed on all proc. [1,num_proc], cause proc. 0 is
                // coordinating everything. So only (num_proc-1) are used.
                for(int i = (p-1)*N/(num_proc-1); i < p*N/(num_proc-1); i++){
                    // printf("Get forces for Obj. %d from proc. %d \n", i, p);
                    sys.Objects[i].ax = a[2*i];
                    sys.Objects[i].ay = a[2*i+1];
                }

            }

            printf("\nStep %d: Obj[3].ax %lf    Obj[3].ay %lf\n\n", step, sys.Objects[3].ax, sys.Objects[3].ay);



            // for(int i = 0; i < N; i++){
            //     sys.update(&sys.Objects[i], delta_t);
            // }

            // ------- update all positions: -----------
            // distribute data
            for(int p = 1; p < num_proc; p++){

                // // OPTION 1:
                // now only send necessary data, so entries with indices
                // [(p-1)*N/(num_proc-1), p*N/(num_proc-1)]
                // max length is 2*(N/(num_proc-1) + 1)
                // // send only proc. - necessary entries:
                // for(int i = (p-1)*N/(num_proc-1); i < p*N/(num_proc-1); i++){
                //     y_buffer[2*i] = sys.Objects[i].x;
                //     y_buffer[2*i+1] = sys.Objects[i].y;
                //     v_buffer[2*i] = sys.Objects[i].u;
                //     v_buffer[2*i+1] = sys.Objects[i].v;
                //     a_buffer[2*i] = sys.Objects[i].ax;
                //     a_buffer[2*i+1] = sys.Objects[i].ay;
                // }
                // MPI_Send(y_buffer, 2*(N/(num_proc-1) + 1), MPI_DOUBLE, p, p, MPI_COMM_WORLD);
                // MPI_Send(v_buffer, 2*(N/(num_proc-1) + 1), MPI_DOUBLE, p, p, MPI_COMM_WORLD);
                // MPI_Send(a_buffer, 2*(N/(num_proc-1) + 1), MPI_DOUBLE, p, p, MPI_COMM_WORLD);

                // Option 2:
                // send all entries to each proc.:
                MPI_Send(y, 2*N, MPI_DOUBLE, p, 0, MPI_COMM_WORLD);
                MPI_Send(v, 2*N, MPI_DOUBLE, p, 0, MPI_COMM_WORLD);
                MPI_Send(a, 2*N, MPI_DOUBLE, p, 0, MPI_COMM_WORLD);
            }

            // update all position:
            // receive all calculated pos and vel and pxl for the timestep:
            for(int p = 1; p < num_proc; p++){
                MPI_Recv(y, 2*N, MPI_DOUBLE, p, 0,
                    MPI_COMM_WORLD, MPI_STATUS_IGNORE);
                MPI_Recv(v, 2*N, MPI_DOUBLE, p, 0,
                    MPI_COMM_WORLD, MPI_STATUS_IGNORE);
                MPI_Recv(a, 2*N, MPI_DOUBLE, p, 0,
                    MPI_COMM_WORLD, MPI_STATUS_IGNORE);
                MPI_Recv(pxl, 2*N, MPI_DOUBLE, p, 0,
                    MPI_COMM_WORLD, MPI_STATUS_IGNORE);
                // write this into forces of the Obj. N/num_proc Obj.:
                // work was distributed on all proc. [1,num_proc], cause proc. 0 is
                // coordinating everything. So only (num_proc-1) are used.
                for(int i = (p-1)*N/(num_proc-1); i < p*N/(num_proc-1); i++){
                    // printf("Get forces for Obj. %d from proc. %d \n", i, p);
                    sys.Objects[i].x = y[2*i];
                    sys.Objects[i].y = y[2*i+1];
                    sys.Objects[i].u = v[2*i];
                    sys.Objects[i].v = v[2*i+1];
                    sys.Objects[i].ax = a[2*i];
                    sys.Objects[i].ay = a[2*i+1];
                    sys.Objects[i].pxl_x = pxl[2*i];
                    sys.Objects[i].pxl_y = pxl[2*i+1];
                }
            }
            // ------- finished updating positions ------- //
            for(int i = 0; i < sys.N; i++){
                sys.write_orbit(&sys.Objects[i]);
            }

        }

        // don't burn up the CPU
        // for real time simulations, use:
        // if((1000*delta_t) > (SDL_GetTicks()-anim.start)){
        //     SDL_Delay((1000*delta_t) - (SDL_GetTicks() - anim.start)); //in milli-seconds [ms]
        // }
        // fps definition, use:
        // SDL_Delay(int(1000.0/speed));    // 1000, because [ms] and fps in [s]

        // for testing, stop after MAX_SETPS iterations:
        if(t > MAX_SETPS*delta_t){
            anim.done = 1;
        }

        // send status to all procs.
        for(int p = 1; p < num_proc; p++){
            MPI_Send(&anim.done, 1, MPI_INT, p, 0, MPI_COMM_WORLD);
        }

        printf("End of step = %d\n", step);
        step++;;
        sys.timestep++;;
    }


    // is that necessary? Or does MPI_Finalize finish all processes?
    // for i in [1,Np]:
    //     MPI_Send(done, 1, MPI_INT, i)
    anim.destroy();

    // free(y_buffer);
    // free(v_buffer);
    // free(a_buffer);
    }


    // all procs. which are not zero (cause zeros coordinates sim) calculate:
    if(rank != 0){

    // int p;  // stores calculated index of proc. to send data to.

    // use declaration of system (weiter oben geschehen), in den Daten aus
    // den empfangenen Zustandsvektoren einsortiert werden kann
    // methods attraction and Euler are available here.

    // send and receive as long as rank zero does not tell sim is done:

    int done = 0;
    while(!done){

        // ----------- update accel: ---------
        printf("Proc %d: wating for y and mass. \n", rank);


        MPI_Recv(y, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
        MPI_Recv(mass, N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE);

        printf("Proc %d: received y and mass. \n", rank);

        // einsortieren von Zustandsvektor in system
        for(int i = 0; i < N; i++){
            sys.Objects[i].x = y[2*i];
            sys.Objects[i].y = y[2*i+1];
            sys.Objects[i].mass = mass[i];
        }


        printf("[%d] x[3] = %lf and y[3] = %lf \n", rank, sys.Objects[3].x, sys.Objects[3].y);

        // update accelerations using sys.attratcion Routine:
        // only update accelerations of the N/(num_proc-1) Objects that the
        // current processor proc. = rank is in charge:
        for(int i = (rank-1)*N/(num_proc-1); i < rank*N/(num_proc-1); i++){
            // printf("Calc. attr. for Obj. %d from proc. %d \n", i, rank);
            sys.attraction(&sys.Objects[i]);
            // write calculated accelerations of Object i in state vector:
            a[2*i] = sys.Objects[i].ax;
            a[2*i + 1] = sys.Objects[i].ay;
        }

        printf("Proc. %d: send a to proc. zero.\n", rank);
        MPI_Send(a, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD);



        // ---------- update pos. ---------- //

        // receive data:
        MPI_Recv(y, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
        MPI_Recv(v, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
        MPI_Recv(a, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE);

        // einsortieren von Zustandsvektor in system:
        for(int i = 0; i < N; i++){
            sys.Objects[i].x = y[2*i];
            sys.Objects[i].y = y[2*i+1];
            sys.Objects[i].u = v[2*i];
            sys.Objects[i].v = v[2*i+1];
            sys.Objects[i].ax = a[2*i];
            sys.Objects[i].ay = a[2*i+1];
        }
        // >>> Euler SChritt
        // perform Euler step only for the N/(num_proc-1) Objects that the
        // current processor proc. = rank is in charge:
        for(int i = (rank-1)*N/(num_proc-1); i < rank*N/(num_proc-1); i++){
            // printf("Calc. attr. for Obj. %d from proc. %d \n", i, rank);
            sys.update(&sys.Objects[i], delta_t);
            // write calculated accelerations of Object i in state vector:
            y[2*i] = sys.Objects[i].x;
            y[2*i + 1] = sys.Objects[i].y;
            v[2*i] = sys.Objects[i].u;
            v[2*i + 1] = sys.Objects[i].v;
            a[2*i] = sys.Objects[i].ax;
            a[2*i + 1] = sys.Objects[i].ay;
            pxl[2*i] = sys.Objects[i].pxl_x;
            pxl[2*i + 1] = sys.Objects[i].pxl_y;
        }

        // Send results to proc. 0 for visualization and further coordinatiion:
        MPI_Send(y, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD);
        MPI_Send(v, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD);
        MPI_Send(a, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD);
        MPI_Send(pxl, 2*N, MPI_DOUBLE, 0, 0, MPI_COMM_WORLD);

        // recv status from 0:
        MPI_Recv(&done, 1, MPI_INT, 0, 0, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
    }


    }

    // free storage of sys which is allocated on every processor,
    // so every proc. need to free (no if rank ... necessary):
    sys.destroy();


    // free strage of Zustandsvektoren
    free(y);
    free(v);
    free(a);
    free(pxl);
    free(mass);


    // Terminate MPI execution environment
    MPI_Finalize();


    return 0;
}
