#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "system.h"
#include "anim.h"
#include "body.h"


void System::create(){

    Objects->create();

    Np = Nbody;
    Na = NAstroids;
    N = Np + Na;

    // create empty Objects:
   Objects = new Object[N];
   for(int i = 0; i < N; i++){
       Objects[i].init("no_name", 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, BLUE, 10, PLANET);
   }

}


void System::init(){

    timestep = 0;

	//==================================================================//
	// Initialization of OBJECTS with type "SUN, PLANET, ROCKET"     	//
	//==================================================================//

    /* SCALING:
    positions of planets are scaled in units of AU.
    So Planet.x = 1.0 means a distance of 1AU to the origin (0.0,0.0),
    where the sun is.
    mass: mass in multiples of earth masses, so:
    mass of sun = 332948.6
    mass of earth = 1

    init(const char *name, double x0, double y0, double u0,
        double v0, double r, double m, const double *col, int anim_r, Object_class type);

    */

    double vel_earth = 2*M_PI; // velocity of the earth in nondim. units

    //init with reference values of earth [M_earth, a_earth, AU]:
    //init sun:
    Objects[0].init("sun", 0.0, 0.0, 0.0, 0.0, 16.0, 332948.6, YELLOW, 20, SUN);

    //init mercury:
    Objects[1].init("mercury", 0.387 , 0.0, 0.0, 1.592*vel_earth, 0.3829, 0.055, DARK_GREY, 3, PLANET);

    //init venus:
	Objects[2].init("venus", 0.723, 0.0, 0.0, 1.1754*vel_earth, 0.9499, 0.815, LIGHT_GREY, 8, PLANET);

    //init earth:
    Objects[3].init("earth", 1.0, 0.0, 0.0, vel_earth, 16.0, 1.0, BLUE, 10, PLANET);

    // init earth moon:
    Objects[4].init("moon", 1.002569, 0.0, 0.0, 1.0336*vel_earth, 0.2725, 0.0123 , DARK_BLUE, 2, PLANET);

	//init mars:
    Objects[5].init("mars", 1.0904, 1.0904, -0.5728*vel_earth, 0.5728*vel_earth, 0.533, 2250.107, RED, 5, PLANET);

	//init jupiter:
    //Objects[6].init("jupiter", -5.2, 0.0, 0.0, -0.43*vel_earth, 0.533, 317.0, ORANGE, 12, PLANET);
    Objects[6].init("jupiter", -5.0455, 1.2580, -0.1040*vel_earth, -0.41723*vel_earth, 0.533, 377.0, ORANGE, 16, PLANET);

	//init saturn:
	//Objects[7].init("saturn", 0.0, -9.6, 0.3254*vel_earth, 0.0, 9.1402, 95.16, LIGHT_BROWN, 26, PLANET);
	Objects[7].init("saturn", -8.7356, -3.9811, 0.1349*vel_earth, -0.2961*vel_earth, 9.1402, 295.16, LIGHT_BROWN, 14, PLANET);

	//init uranus:
	//Objects[8].init("uranus", -19.39, 0.0, 0.0, -0.2287*vel_earth, 4.007, 14.54, LIGHT_BLUE, 12, PLANET);
	//Objects[8].init("uranus", -13.71, -13.71, 0.1617*vel_earth, -0.1617*vel_earth, 4.007, 14.54, LIGHT_BLUE, 12, PLANET);
	Objects[8].init("uranus", -14.5893, -12.7720, 0.1506*vel_earth, -0.1721*vel_earth, 4.007, 14.54, LIGHT_BLUE, 12, PLANET);

	//init neptune:
	Objects[9].init("neptune", 30.08, 0.0, 0.0, 0.1823*vel_earth, 3.883, 17.15, TURQUOISE, 11, PLANET);

	//init rocket:
	Objects[10].init("Rocket", 1.0 + pow(4.2588, -5), 0.0, 0.01*vel_earth, 1.1981*vel_earth, 7.84*pow(10,6), 5.442*pow(10,-20), PINK, 5, ROCKET);

}


/* To do: Write randomizer for velocities distances, radii of asteroid objects:*/

void System::belt_init(){

	//==================================================================//
	// Initialization of Asteroids and Belt-Objects 					//
	//==================================================================//

	// mean distance and position and dist. variation
	double mean_dist = 2.5;
	double delta_dist = 0.5;
	double dist = 0.0;
	double x = 0.0;
	double y = 0.0;

	// velocity of an asteroid in the asteroid belt
	double vel_earth = 2*M_PI;
	double mean_vel = 0.6351*vel_earth;
	//double mean_vel = 0.0*vel_earth;
	double delta_vel = 0.0*vel_earth;
	double vel = 0.0;
	double vel_x = 0.0;
	double vel_y = 0.0;

	// velocity of an asteroid in the asteroid belt
	double theta = 0.0;

	// mass of an asteroid in the asteroid belt:
	double mean_m = 4*pow(10,-14);
	double delta_m = 2*pow(10,-14);
	double mass = 0.0;

	srand(time(NULL));

	for(int i = Np; i < Na; i++){

		theta = rand_val(0, 360);
		//printf("%f\n", theta);

		dist = mean_dist + rand_val(-delta_dist, delta_dist);
		//printf("%f\n", dist);

		x = sin(theta)*dist;
		y = cos(theta)*dist;

		mean_vel = sqrt((G*M_sun/M_earth)/(dist));

		vel = mean_vel + rand_val(-delta_vel, delta_vel);

		vel_x = cos(theta)*vel;
		vel_y = sin(theta)*vel;

		mass = mean_m + rand_val(-delta_m, delta_m);


		Objects[i].init("no_name", x, y, -vel_x, vel_y, 0.0, 1.0, WHITE, 2, ASTROID);
	}




}

double System::rand_val(double min, double max){

	double random = 0.0;
	double range = 0.0;
	double val = 0.0;


	random = ((double) rand())/RAND_MAX;
	range = (max - min)*random;
	val = min + range;

	return val;


}

void System::status_Rocket(Object *Plt){

    if(Plt->type == ROCKET){

        double rad_dist_rocket = 0;
        double rad_dist_planet = 0;

        rad_dist_rocket = sqrt(Objects[10].x*Objects[10].x + Objects[10].y*Objects[10].y);
        rad_dist_planet = sqrt(Objects[Plt->orbit_transit].x*Objects[Plt->orbit_transit].x + Objects[Plt->orbit_transit].y*Objects[Plt->orbit_transit].y);

        if (rad_dist_rocket >= rad_dist_planet && Objects[Plt->orbit_transit].type != 3){

            printf("Farewell %s, thank you for the boost <3. \n", Objects[Plt->orbit_transit].name);
            printf("Orbit transit = %i\n", Plt->orbit_transit);
            printf("Rad distance rocket-sun: %f\n", rad_dist_rocket);
            printf("Rad distance %s-sun: %f\n", Objects[Plt->orbit_transit].name, rad_dist_planet);
            printf("#==============================================#\n");

            Plt->orbit_transit += 1;
        }
        // printf("%d\n", timestep);
        // printf("%f\n", Objects[Plt->orbit_transit].x);



    }
}

void System::attraction(Object *Plt){

    double force = 0.0;
    double dist_x = 0.0;
    double dist_y = 0.0;
    double dist = 0.0;
    double F = 0.0, F_x = 0.0, F_y = 0.0;

    // set forces to zero and calculate them new:
    for(int i = 0; i < Np; i++){
        Plt->ax = 0.0;
        Plt->ay = 0.0;
    }

    for(int i = 0; i < Np; i++){
        dist_x = Plt->x - Objects[i].x;
        dist_y = Plt->y - Objects[i].y;
        dist = sqrt(dist_x*dist_x + dist_y*dist_y);

        // do not calculate force on body on itself
        // also, because mathematically, you would devide by dist=0
        if(strcmp(Objects[i].name, Plt->name) != 0){
            F = -(G * Plt->mass * Objects[i].mass)/(dist*dist);
            F_x = F * (dist_x/dist);
			F_y = F * (dist_y/dist);

            Plt->ax += F_x/Plt->mass;
            Plt->ay += F_y/Plt->mass;

            // to imporve perfomance, one could also use the calculated
            // F_x and F_y to update forces on the other objects
            // but then they also need to be reset to zero seperately
            // before accumulated
            // Planets[i].fx -= F_x/Planets[i].mass;
            // Planets[i].fy -= F_y/Planets[i].mass;

        }

    }

}

void System::update(Object *Plt, double delta_t){

    // ------ EULER step: ------ //
    Plt->u += Plt->ax * delta_t;
    Plt->v += Plt->ay * delta_t;

    Plt->x += Plt->u * delta_t;
    Plt->y += Plt->v * delta_t;

    // calc position in rendering:
    Plt->pxl_x = ORIGx + scale*Plt->x;
    Plt->pxl_y = ORIGy - scale*Plt->y;


}

void System::write_orbit(Object *Plt){
	//Save Points of the planets in order to draw the orbit
    //Failsafe --> timestep > len(orbit_x/orbit_y):

    //Don't draw lines for farfield cases, very confusing in the central portion of the solar system
    //if(Plt-> name != "venus" && Plt-> name != "mercury" && Plt-> name != "moon"){
    if(timestep > Plt->orbit_size){
        printf(">>> Orbits filled completaly! \n");
    }
    else{
    	Plt->orbit_x[timestep] = Plt->pxl_x;
    	Plt->orbit_y[timestep] = Plt->pxl_y;
    }

}


void System::destroy(){

    // delete varaibles

    // delete [] a;

    Objects->destroy();
    free(Objects);
}
