#ifndef SYSTEM_H
#define SYSTEM_H

#include "body.h"

// VISUALIZATION VALUES

const int YELLOW[3] = {255, 255, 0};
const int DARK_GREY[3] = {80, 78, 81};
const int LIGHT_GREY[3] = {211, 211, 211};
const int BLUE[3] = {100, 149, 237};
const int DARK_BLUE[3] = {0, 0, 255};
const int RED[3] = {188, 39, 50};
const int ORANGE[3] = {255, 165, 0};
const int LIGHT_BROWN[3] = {244, 217, 146};
const int LIGHT_BLUE[3] = {173, 216, 230};
const int TURQUOISE[3] = {64, 224, 208};
const int WHITE[3] = {255, 255, 255};
const int PINK[3] = {255, 8, 127};

// PHYSICAL VALUES
const int Nbody = 11;
const int NAstroids = 200;
const int Ntot = Nbody + NAstroids;
const double AU = 1.496e11;             // Astronomical unit
const double a = 3.169e7;
const double M_earth = 5.972e24;
const double M_sun = 332948.6*M_earth;

const double G = (6.674e-11)*M_earth*a*a/(AU*AU*AU);    // Gravitational constant
// const double G = 6.674e-11;    // Gravitational constant
// const double SCALE = 200.0/AU;             // 1AU = 100 Pixels
const double TIMESTEP = 3600.0*24.0;         // Timestep = 1 day





class System{
    public:

    int Np;     // number of planets
    int Na;		// number of Astroids
    int N;

    int timestep;


    Object *Objects;

    void create();
    void init();
    void destroy();

    void belt_init();
    double rand_val(double min, double max);
    void status_Rocket(Object *Plt);

    void attraction(Object *);
    void update(Object *, double delta_t);
    void write_orbit(Object *Plt);

};

#endif
